<?php
/**
 * The public plugin class.
 *
 * @since      1.0
 * @package    ICS_Display
 */

defined( 'ABSPATH' ) || exit;

class ICS_Display_Public {
    
    private  $plugin_name ;
    
    /* init class */
    public function __construct( $plugin_name ) {
        $this->plugin_name = $plugin_name;
    }
    
    /* public area css */
    public function enqueue_styles() {
        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url( __DIR__ ) . 'css/icsd-public.css'
        );
    }
    
    /* public area javascript */
    public function enqueue_scripts() {}
    
    public function register_shortcodes() {
        add_shortcode( 'icsd', array($this, 'ics_display') );
    }
    
    public function get_ics_data ( $url ) {
        
        $args['timeout'] = 30;
        
        $response = wp_remote_get( $url , $args );
        $http_code = wp_remote_retrieve_response_code( $response );
        
        if ($http_code==200) {
            $content = wp_remote_retrieve_body( $response );
            $icsdata = explode(PHP_EOL, $content);
            $act = 0;
            if (is_array($icsdata)) {
                foreach($icsdata AS $ik => $iv) {
                    if ($ik>0 && intval(strpos($iv, ":"))==0) {
                        if (isset($icsdata[$act])) {
                            $icsdata[$act] = trim($icsdata[$act]).trim($iv);
                        }
                        unset($icsdata[$ik]);
                    }
                    $act = $ik;
                }
                $events = array(); $e = 0; $timezone = date_default_timezone_get();
                foreach ($icsdata AS $ik => $iv) {
                    if (strstr(trim($iv), 'X-WR-TIMEZONE:')) { 
                        list(,$timezone) = explode(":",trim($iv)); 
                    }
                    if (strstr(trim($iv), 'BEGIN:VEVENT')): $events[$e] = array(); $events[$e]['timezone'] = $timezone; endif;
                    if (strstr(trim($iv), 'DTSTART:')): 
                        // if dtstart empty or not given, it seems to be a full day entry
                        list(,$events[$e]['dtbegin']) = explode(":",trim($iv));
                        // convert format to "normal" timestamp
                        $events[$e]['dtbegin'] = mktime(substr($events[$e]['dtbegin'],9,2),substr($events[$e]['dtbegin'],11,2),substr($events[$e]['dtbegin'],13,2),substr($events[$e]['dtbegin'],4,2),substr($events[$e]['dtbegin'],6,2),substr($events[$e]['dtbegin'],0,4));
                        $events[$e]['type'] = 'daytime';
                    endif;
                    if (strstr(trim($iv), 'DTEND:')): 
                        list(,$events[$e]['dtend']) = explode(":",trim($iv));
                        $events[$e]['dtend'] = mktime(substr($events[$e]['dtend'],9,2),substr($events[$e]['dtend'],11,2),substr($events[$e]['dtend'],13,2),substr($events[$e]['dtend'],4,2),substr($events[$e]['dtend'],6,2),substr($events[$e]['dtend'],0,4));
                    endif;
                    if (strstr(trim($iv), 'DTSTART;')):
                        list(,$events[$e]['dtbegin']) = explode(":",trim($iv));
                        $events[$e]['dtbegin'] = mktime(0,0,0,substr($events[$e]['dtbegin'],4,2),substr($events[$e]['dtbegin'],6,2),substr($events[$e]['dtbegin'],0,4));
                        $events[$e]['type'] = 'day';
                    endif;
                    if (strstr(trim($iv), 'DTEND;')):
                        list(,$events[$e]['dtend']) = explode(":",trim($iv));
                        $events[$e]['dtend'] = mktime(0,0,-1,substr($events[$e]['dtend'],4,2),substr($events[$e]['dtend'],6,2),substr($events[$e]['dtend'],0,4));
                    endif;
                    if (strstr(trim($iv), 'DTSTAMP:')): 
                        list(,$events[$e]['dtstamp']) = explode(":",trim($iv)); 
                        $events[$e]['dtstamp'] = mktime(substr($events[$e]['dtstamp'],9,2),substr($events[$e]['dtstamp'],11,2),substr($events[$e]['dtstamp'],13,2),substr($events[$e]['dtstamp'],4,2),substr($events[$e]['dtstamp'],6,2),substr($events[$e]['dtstamp'],0,4));
                        $events[$e]['converted'] = time();
                    endif;
                    if (strstr(trim($iv), 'CREATED:')): 
                        list(,$events[$e]['created']) = explode(":",trim($iv));
                        $events[$e]['timeformat'] = substr($events[$e]['created'],15,1);
                        $events[$e]['created'] = mktime(substr($events[$e]['created'],9,2),substr($events[$e]['created'],11,2),substr($events[$e]['created'],13,2),substr($events[$e]['created'],4,2),substr($events[$e]['created'],6,2),substr($events[$e]['created'],0,4));
                    endif;
                    if (strstr(trim($iv), 'DESCRIPTION:')) {
                        list(,$events[$e]['description']) = explode(":",trim($iv));
                    }
                    if (strstr(trim($iv), 'LOCATION:')) { 
                        list(,$events[$e]['location']) = explode(":",trim($iv));
                        $events[$e]['location'] = str_replace('  ', ' ', str_replace('  ', ' ', str_replace("\,", ", ", $events[$e]['location'])));
                    }
                    if (strstr(trim($iv), 'SUMMARY:')): list(,$events[$e]['summary']) = explode(":",trim($iv)); endif;
                    // find final col and start over to new event element
                    if (strstr(trim($iv), 'END:VEVENT')) {
                        ksort($events[$e]);
                        $e++;
                    }
                }
            }
            return ($events);
        }
        else {
            return false;
        }
        
    }
    
    public function ics_get_date ( $start , $end , $timeformat ) {
        if (date('Y-m-d', $start)!=date('Y-m-d', $end)) {
            return date_i18n(get_option('date_format'), $start)." - ".date_i18n(get_option('date_format'), $end);
        } else {
            return date_i18n(get_option('date_format'), $start);
        }
    } 
    
    public function ics_get_time ( $start , $end , $timeformat ) {
        if ($start<($end-3600)) {
            return date(get_option('time_format'), $start)." - ".date(get_option('time_format'), $end);
        } else if (date(get_option('time_format'), $start)!='00:00') {
            return date(get_option('time_format'), $start);
        } else {
            return '-';
        }
    } 
    
    public function ics_display_table ( $data ) {
        
        $thead = explode(',',$data['atts']['dataset']);
        $thdesc = explode(',',$data['atts']['header']);
        if (is_array($thead) && count($thead)>0) {
            $cols = count($thead);
            $header = array();
            /* prepare header description */
            foreach ($thead AS $tk => $tv) {
                $header[$tv] = (isset($thdesc[$tk]) && trim($thdesc[$tk])!='')?trim($thdesc[$tk]):__($tv, $this->plugin_name);
            }
        } else {
            $cols = false;
        }
        
        if (!isset($data['show']) || !is_array($data['show']) || count($data['show'])<1) { $cols = false; }
        if (!isset($data['list']) || !is_array($data['list']) || count($data['list'])<1) { $cols = false; }
        
        $return = '';
        $idselector = md5(rand(0,time()));
        if ($cols!==false) {
            $return.= '<table class="icsd-table" id="icsd-table-'.$idselector.'">';
            $return.= '<thead>';
            $return.= '<tr>';
            $return.= '<th class="icsd-numbering"></th>';
            foreach ($thead AS $tk => $tv) {
                $return.= '<th class="icsd-th icsd-th-'.trim($tv).'">'.(isset($header[$tv])?$header[$tv]:'').'</th>';
            }
            $return.= '</tr>';
            $return.= '</thead>';
            $return.= '<tbody>';
            $item = $page = $num = 0;
            foreach ($data['show'] AS $dsk => $dsv) {
                foreach ($dsv AS $dk => $dv) {
                    $num++;
                    $return.= '<tr class="icsd-page icsd-page-'.$page.'">';
                    $return.= '<td class="icsd-numbering">'.$num.'</td>';
                    foreach ($thead AS $tk => $tv) {
                        $return.= '<td class="icsd-td-'.trim($tv).'">'.(isset($data['list'][$dv][$tv])?$data['list'][$dv][$tv]:'-').'</td>';
                    }
                    $return.= '</tr>';
                    $item++;
                    if ($data['atts']['page']>0 && $item>=$data['atts']['page']) {
                        $item = 0;
                        $page++;
                    }
                }
            }
            $return.= '</tbody>';
            $return.= '</table>';
            if ($page>0) {
                $return.= '<ul id="icsd-pager-'.$idselector.'" class="icsd-pager">';
                for ($p=0; $p<$page; $p++) {
                    $return.= '<li rel="'.$p.'" class="icsd-pager-item '.(($p==0)?'active':'').'">'.($p+1).'</li>';
                }
                $return.= '</ul>';
            }
            $return.= '<style>
            
            #icsd-table-'.$idselector.' .icsd-page { display: none; }
            #icsd-table-'.$idselector.' .icsd-page-0 { display: table-row; }

            </style>';
            $return.= '<script>
            
    jQuery(\'#icsd-pager-'.$idselector.' .icsd-pager-item\').on(\'click\', function() {
        jQuery(\'#icsd-pager-'.$idselector.'\').find(\'.icsd-pager-item\').removeClass(\'active\');
        jQuery(this).addClass(\'active\');
        jQuery(\'#icsd-table-'.$idselector.'\').find(\'.icsd-page\').hide();
        jQuery(\'#icsd-table-'.$idselector.'\').find(\'.icsd-page-\' + jQuery(this).attr(\'rel\')).css(\'display\', \'table-row\');
    });
            
            </script>';
        }
        
        return $return;
    }
    
    public function ics_display_list ( $data ) {
        
        $thead = explode(',',$data['atts']['dataset']);
        $thdesc = explode(',',$data['atts']['header']);
        if (is_array($thead) && count($thead)>0) {
            $cols = count($thead);
            $header = array();
            /* prepare header description */
            foreach ($thead AS $tk => $tv) {
                $header[$tv] = (isset($thdesc[$tk]) && trim($thdesc[$tk])!='')?trim($thdesc[$tk]):__($tv, $this->plugin_name);
            }
        } else {
            $cols = false;
        }
        
        if (!isset($data['show']) || !is_array($data['show']) || count($data['show'])<1) { $cols = false; }
        if (!isset($data['list']) || !is_array($data['list']) || count($data['list'])<1) { $cols = false; }
        
        $return = '';
        $idselector = md5(rand(0,time()));
        if ($cols!==false) {
            $return.= '<ol class="icsd-list" id="icsd-list-'.$idselector.'">';
            $item = $page = $num = 0;
            foreach ($data['show'] AS $dsk => $dsv) {
                foreach ($dsv AS $dk => $dv) {
                    $num++;
                    $return.= '<li class="icsd-page icsd-page-'.$page.'"><ul class="icsd-list-itemset">';
                    foreach ($thead AS $tk => $tv) {
                        if (isset($data['list'][$dv][$tv]) && trim($data['list'][$dv][$tv])!='') {
                            $return.= '<li class="icsd-td-'.trim($tv).'">';
                            $return.= (isset($header[$tv])?'<span class="icsd-item-desc icsd-item-desc-'.$tv.'">'.$header[$tv].': </span>':'');
                            $return.= $data['list'][$dv][$tv];
                            $return.= '</li>';
                        }
                    }
                    $return.= '</ul></li>';
                    $item++;
                    if ($data['atts']['page']>0 && $item>=$data['atts']['page']) {
                        $item = 0;
                        $page++;
                    }
                }
            }
            $return.= '</ol>';
            if ($page>0) {
                $return.= '<ul id="icsd-pager-'.$idselector.'" class="icsd-pager">';
                for ($p=0; $p<$page; $p++) {
                    $return.= '<li rel="'.$p.'" class="icsd-pager-item '.(($p==0)?'active':'').'">'.($p+1).'</li>';
                }
                $return.= '</ul>';
            }
            $return.= '<style>
            
            #icsd-list-'.$idselector.' .icsd-page { display: none; }
            #icsd-list-'.$idselector.' .icsd-page-0 { display: list-item; }

            </style>';
            $return.= '<script>
            
    jQuery(\'#icsd-pager-'.$idselector.' .icsd-pager-item\').on(\'click\', function() {
        jQuery(\'#icsd-pager-'.$idselector.'\').find(\'.icsd-pager-item\').removeClass(\'active\');
        jQuery(this).addClass(\'active\');
        jQuery(\'#icsd-list-'.$idselector.'\').find(\'.icsd-page\').hide();
        jQuery(\'#icsd-list-'.$idselector.'\').find(\'.icsd-page-\' + jQuery(this).attr(\'rel\')).css(\'display\', \'list-item\');
    });
            
            </script>';
        }
        
        return $return;
    }
    
    public function ics_display_calendar_sheet ( $data ) {
        // for different starting days of week
        $wd = array('Sun','Mon','Tue','Wed','Thu','Fri','Sat','Sun','Mon','Tue','Wed','Thu','Fri');
        // begin output
        $sheet = '<table class="icsd-cal-table">';
        $sheet.= '<thead>';
        $sheet.= '<tr class="icsd-cal-table-month"><th colspan="7">'.date_i18n('F', $data['fd']).'</th></tr>';
        $sheet.= '<tr>';
        // run $wd to display weekdays with correct beginning 
        for ($sw=get_option('start_of_week'); $sw<(get_option('start_of_week')+7); $sw++) {
            $sheet.= '<th>'.__($wd[$sw]).'</th>';
        }
        $sheet.= '</tr>';
        $sheet.= '</thead>';
        // day difference -> how much days from start of week will go on until First Day of month
        $ddiff = (intval(date('w', $data['fd']))+7-get_option('start_of_week')) % 7;
        // first day of calendar
        $fdoc = $data['fd']-(86400*$ddiff);
        // weeks in calendar
        $wic = date('t', $data['fd'])+$ddiff; while ($wic % 7 != 0) { $wic++; } $wic = $wic/7;
        $sheet.= '<tbody>';
        for ($w=0; $w<$wic; $w++) {
            $sheet.= '<tr>';
            for ($d=0; $d<7; $d++) {
                $actd = $fdoc+(86400*$d)+(86400*7*$w);
                $class = array();
                $title = array();
                // if month doesn't match
                if (date('F', $actd)!=date('F', $data['fd'])) { $class[] = 'false-month'; }
                foreach ($data['show'] AS $dsk => $dsv) {
                    if ($dsk>=$actd && $dsk<=($actd+86399)) {
                        $class[] = ' active-date ';
                        foreach ($dsv AS $dlk => $dlv) {
                            if ($data['list'][$dlv]['event']!='') {
                                $title[] = $data['list'][$dlv]['event'];
                            }
                        }
                    }
                }
                $sheet.= '<td '.((count($class)>0)?'class="'.implode(' ', $class).'"':'').' '.((count($title)>0)?'title="'.implode('\n', $title).'"':'').'>'.date('j', $actd).'</td>';
            }
            $sheet.= '</tr>';
        }
        $sheet.= '</tbody>';
        $sheet.= '</table>';
        return $sheet;
    }
    
    public function ics_display_calendar ( $data ) {
        
        $thead = explode(',',$data['atts']['dataset']);
        $thdesc = explode(',',$data['atts']['header']);
        if (is_array($thead) && count($thead)>0) {
            $cols = count($thead);
            $header = array();
            /* prepare header description */
            foreach ($thead AS $tk => $tv) {
                $header[$tv] = (isset($thdesc[$tk]) && trim($thdesc[$tk])!='')?trim($thdesc[$tk]):__($tv, $this->plugin_name);
            }
        } else {
            $cols = false;
        }
        
        if (!isset($data['show']) || !is_array($data['show']) || count($data['show'])<1) { $cols = false; }
        if (!isset($data['list']) || !is_array($data['list']) || count($data['list'])<1) { $cols = false; }
        
        $return = '';
        $idselector = md5(rand(0,time()));
        if ($cols!==false) {
            // last check for displaying calendar pages 
            if ($data['atts']['page']==0 || $data['atts']['page']>3) { $data['atts']['page'] = 3; }
            for ($cs=0; $cs<$data['atts']['page']; $cs++) {
                $fd = mktime(0,0,0,date('m',time()+86400)+$cs,1,date('Y',time()+86400));
                $ld = mktime(23,59,59,date('m',time()+86400)+$cs+1,0,date('Y',time()+86400));
                $return.= $this->ics_display_calendar_sheet( array(
                    'fd' => $fd,
                    'ld' => $ld,
                    'list' => $data['list'],
                    'show' => $data['show'],
                ) );
            }
        }
        return $return;
    }
    
    public function ics_display ( $atts ) {
        
        /* get the urls to grep data */
        $calurl = maybe_unserialize( get_option('icsd_url') );
        $baseid = false;
        if ($calurl!==false) {
            foreach ($calurl AS $ck => $cv) {
                if ($baseid===false && $cv!==false) {
                    $baseid = intval($ck);
                }
            }
        }
        
        $atts = shortcode_atts( array(
            'id' => ($baseid+1),
            'max' => false,
            'page' => false,
            'sort' => false,
            'display' => false,
            'dataset' => false,
            'header' => false
        ), $atts);
        
        $defaults = array(
            'max' => 0,
            'page' => 10, // list and table view page is items per page | calendar is months per view
            'sort' => 'asc',
            'display' => 'table',
            'dataset' => 'date,time,event,location',
            'header' => __('date', $this->plugin_name).','.__('time', $this->plugin_name).','.__('event', $this->plugin_name).','.__('location', $this->plugin_name)
        );
        
        /* get all different ID's to get URL from */
        $getid = explode(',', $atts['id']);
        $geturl = array();
        if (is_array($getid) && count($getid)>0) {
            foreach ($getid AS $gk => $gv) {
                if (isset($calurl[($gv-1)]) && trim($calurl[($gv-1)])!='') {
                    $geturl[] = $calurl[($gv-1)];
                }
            }
            if (count($getid)==1) {
                // if there is only a call to ONE shortcode
                $calatts = maybe_unserialize( get_option('icsd_setup') );
                if (isset($calatts[(intval($getid[0])-1)])) {
                    // explode the array holding the config data
                    foreach ($calatts[(intval($getid[0])-1)]['dataset'] AS $cdk => $cdv) {
                        $tmpdataset[] = $cdk;
                        $tmpheader[] = $cdv;
                    }
                    $calatts[(intval($getid[0])-1)]['id'] = intval($getid[0]);
                    $calatts[(intval($getid[0])-1)]['dataset'] = implode(',', $tmpdataset);
                    $calatts[(intval($getid[0])-1)]['header'] = implode(',', $tmpheader);
                    // replace config attributes with shortcode attributes
                    foreach ($calatts[(intval($getid[0])-1)] AS $cak => $cav) {
                        $calatts[(intval($getid[0])-1)][$cak] = (isset($atts[$cak]) && $atts[$cak]!==false)?$atts[$cak]:$cav;
                    }
                }
                $atts = $calatts[(intval($getid[0])-1)];
            }
        }
        // run atts against defaults to insert missing values
        foreach ($defaults AS $dk => $dv) {
            if (!isset($atts[$dk])) {
                $atts[$dk] = $dv;
            }
        }
        
        /* setup the event list to display */
        $eventlist = array();
        foreach ($geturl AS $guk => $guv) {
            $eventlist = array_merge($eventlist, $this->get_ics_data($guv));
        }
        /* run FULL eventlist against array schema to have identical values */
        foreach ($eventlist AS $ek => $ev) {
            ksort($ev);
            if (!(isset($ev['converted']))) { $eventlist[$ek]['converted'] = time(); }
            if (!(isset($ev['created']))) { $eventlist[$ek]['created'] = time(); }
            if (!(isset($ev['description']))) { $eventlist[$ek]['description'] = ''; }
            // dtbegin
            if (!(isset($ev['dtend']))) { $eventlist[$ek]['dtend'] = (isset($ev['dtbegin']))?intval($ev['dtbegin'])+3600:false; }
            // dtstamp
            if (!(isset($ev['location']))) { $eventlist[$ek]['location'] = ''; }
            if (!(isset($ev['summary']))) { $eventlist[$ek]['summary'] = (isset($ev['description']) && trim($ev['description'])!='')?trim($ev['description']):''; }
            if (!(isset($ev['timeformat']))) { $eventlist[$ek]['timeformat'] = 'daytime'; }
            // timezone
            // type
            if (!(isset($ev['dtbegin']))) { 
                unset($eventlist[$ek]);
            }
        }
        // renumber eventlist to _berücksichtigen_ removed entries
        $eventlist = array_values($eventlist);
        /* create a list with all id to startdate */
        $showlist = array();
        foreach ($eventlist AS $ek => $ev) {
            /* convert single entries to some usable data and add some entries */
            $eventlist[$ek]['date'] = $this->ics_get_date( $ev['dtbegin'], $ev['dtend'], $ev['timeformat'] );
            $eventlist[$ek]['dtbegin'] = date_i18n(get_option('date_format'), $ev['dtbegin']);
            $eventlist[$ek]['dtend'] = date_i18n(get_option('date_format'), $ev['dtend']);
            $eventlist[$ek]['dtstamp'] = date_i18n(get_option('date_format'), $ev['dtstamp']);
            $eventlist[$ek]['time'] = $this->ics_get_time( $ev['dtbegin'], $ev['dtend'], $ev['timeformat'] );
            $eventlist[$ek]['event'] = $ev['summary'];
            $showlist[$ev['dtbegin']][] = $ek;
        }
        // do asc sorting to prepare for cutting the array
        ksort($showlist);
        // reduce array to $atts['max'] entries if display is list or table
        if ($atts['display']=='table' || $atts['display']=='list') {
            foreach ($showlist AS $dk => $dv) {
                if ($dk<mktime(0,0,0,date('m'),date('d'),date('Y'))) {
                    unset($showlist[$dk]);
                }
            }
            if (intval($atts['max'])>0) {
                $showlist = array_slice($showlist, 0, (intval($atts['max'])-1), true);
            }
        }
        // sort $showlist it in sorting order given by $atts['sort']
        if ($atts['sort']=='desc') { krsort($showlist); } else { ksort($showlist); }
        // prepare array to give it to display classes
        $data = array(
            'atts' => $atts,
            'list' => $eventlist,
            'show' => $showlist
        );
        // show return by display-type if there is something to show
        // show list and calendar only in PRO
        if (count($showlist)>0) {
            if ($atts['display']=='list') {
                if (defined('ICSD_PRO') && ICSD_PRO) {
                    return $this->ics_display_list($data);
                } else {
                    return __('ICSD PRO License needed', $this->plugin_name);
                }
            }
            else if ($atts['display']=='calendar') {
                if (defined('ICSD_PRO') && ICSD_PRO) {
                    return $this->ics_display_calendar($data);
                } else {
                    return __('ICSD PRO License needed', $this->plugin_name);
                }
            }
            else {
                return $this->ics_display_table($data);
            }
        }
    }
    
}